﻿Type=StaticCode
Version=1.50
@EndOfDesignText@
'Code module
'Subs in this code module will be accessible from all modules.
Sub Process_Globals
	Dim vx, vy, maxLeft, maxDown As Float
	Dim BallRect As Rect
	Dim BallSize As Int
	Dim maxVx As Float
End Sub
Sub Reset (width As Int, height As Int, Drawers As DrawerType)
	BallSize = Ceil(5 * (1%x +  1%y) / 2)
	BallRect.Initialize(50%x - BallSize / 2, height - BallSize, 50%x + BallSize / 2, height)
	vx = 1.2%x
	vy = -1.2%y
	maxVx = 1.5 * vx
	maxLeft = width
	maxDown = height
	Drawers.canvasBall.DrawCircle(BallRect.CenterX, BallRect.CenterY, BallSize / 2, Colors.Blue, True, 0)
End Sub
Sub Tick (Drawers As DrawerType)
	'Erase previous ball
	Drawers.canvasBall.DrawRect(BallRect, Colors.Transparent, True, 0)
	Drawers.pnlBall.Invalidate2(BallRect)
	'Check if the ball hit anything.
	'It will return true if the game ends 
	If Hit(Drawers) Then Return
	If BallRect.Right + vx >= maxLeft Then
		PlayBounce
		vx = -Abs(vx)
	Else If BallRect.Left + vx <= 0 Then
		PlayBounce
		vx = Abs(vx)
	End If
	If BallRect.Bottom + vy >= maxDown Then
		PlayBounce
		vy = -Abs(vy)
	Else If BallRect.Top + vy <= 0 Then
		PlayBounce
		vy = Abs(vy)
	End If
	BallRect.Left = BallRect.Left + vx
	BallRect.Top = BallRect.Top + vy
	BallRect.Right = BallRect.Left + BallSize
	BallRect.Bottom = BallRect.Top + BallSize
	'draw new ball
	Drawers.canvasBall.DrawCircle(BallRect.CenterX, BallRect.CenterY, BallSize / 2, Colors.Blue, True, 0)
	Drawers.pnlBall.Invalidate2(BallRect)
End Sub
Sub PlayBounce
	'play the bounce sound with a priority of 0 (the lowest priority)
	Main.sounds.Play(Main.bounceId, 1, 1, 0, 0, 1)
End Sub
Sub Hit(Drawers As DrawerType) As Boolean
	'Check if the ball hits the slider
	If BallRect.Bottom + vy >= maxDown Then
		If BallRect.Right + vx < Slider.x - Slider.SliderWidth OR BallRect.Left + vx > Slider.x + Slider.SliderWidth  Then
			'The slider is not in its position...
			Main.sounds.Play(Main.crashId, 1, 1, 2, 0, 2)
			CallSub2("Main", "GameOver", "Game Over")
			Return True 'Prevent the drawing of the ball
		Else
			vx = vx + 0.05 * Slider.vx 'slightly change the velocity of the ball if the slider is moving.
			If Abs(vx) > maxVx Then vx = Abs(vx) / vx * maxVx
			PlayBounce
		End If
	End If
	Dim brickX, brickY, brickToBreakX, brickToBreakY As Int
	brickToBreakX = -1 '-1 means that we haven't found yet a corner that intersects with a brick
	Dim x(2), y(2) As Int
	Dim corners(2,2) As Boolean
	Dim countCorners As Int
	x(0) = BallRect.Left + vx
	x(1) = BallRect.Right + vx
	y(0) = BallRect.Top + vy
	y(1) = BallRect.Bottom + vy
	'Find if a brick was hit and by which corners.
	For ix = 0 To 1
		brickX = x(ix) / Bricks.BrickWidth
		For iy = 0 To 1
			brickY = y(iy) / Bricks.BrickHeight
			If brickToBreakX = -1 OR (brickX = brickToBreakX AND brickY = brickToBreakY) Then
				If brickY < Bricks.numY AND brickX < Bricks.numX AND Bricks.BricksStatus(brickX, brickY) = True Then
					brickToBreakX = brickX
					brickToBreakY = brickY
					corners(ix, iy) = True
					countCorners = countCorners + 1
				End If
			End If
		Next
	Next
	If countCorners > 0 Then
		'Break the brick. If it is the last one then return true
		If Bricks.BreakBrick(brickToBreakX, brickToBreakY, Drawers) Then Return True
		If countCorners = 1 Then
			'this is more complex as only one corner hits the brick.
			'we need to check the previous position to decide how to change the velocity.
			For ix = 0 To 1
				For iy = 0 To 1
					If corners(ix, iy) = True Then
						'check the position before the hit
						brickX = (x(ix) - vx) / Bricks.BrickWidth
						brickY = (y(iy) - vy) / Bricks.BrickHeight
						If brickX = brickToBreakX Then
							vy = - vy
						Else If brickY = brickToBreakY Then
							vx = -vx
						Else
							vy = -vy
						End If
					End If
				Next
			Next
		Else If countCorners > 1 Then
			If corners(0, 0) = True AND corners(1, 0) = True Then
				'top left corner and top right corner
				vy = -vy
			Else
				vx = -vx
			End If
		End If
	End If
	Return False
End Sub